class PORTL::Engine
  include ActionView::Helpers::TagHelper

  attr_reader :data, :scope, :options

  attr_accessor :output_buffer

  def initialize(data, scope=nil, options={})
    @data = data
    @scope = scope
    @options = options
    @output_buffer = ActionView::OutputBuffer.new
  end

  # Compiles the data within the supplied scope and returns the result.
  #
  # @return [String] The result of the compiled code.
  def result
    eval(data)
  end

  # Creates an HTML tag.
  #
  # @param [Symbol] tag The tag to generate.
  # @param [*Array] args Arguments to pass to the Rails `content_tag` helper.
  # @param [&Proc] block Code to render inside of the tag.
  # @param [ActionView::OutputBuffer] The updated buffer.
  def html_tag(tag, *args, &block)
    if PORTL::HTML::NON_CLOSING_TAGS.include?(tag)
      output_buffer << raw("<#{tag}#{tag_options(args[0], true)}>")
    else
      output_buffer << content_tag(tag, *args, &block)
    end
  end

  # Adds plain text to the output buffer.
  #
  # @param [String] value The text to add.
  # @return [ActionView::OutputBuffer] The updated buffer.
  def text(value)
    output_buffer << raw(value)
  end

  private

  # Dynamically create tag generators for all of the supported HTML tags.
  PORTL::HTML::TAGS.each do |tag|
    # Do not redefine a method if it already exists.
    next if methods.include?(tag)

    # Delegate to the #html_tag method, passing the tag as the first argument.
    define_method(tag) {|*args, &block| html_tag(tag, *args, &block) }
  end
end
